@@ -4,7 +4,7 @@
 #'
 #' @author Jose M. Pavia, \email{pavia@@uv.es}
 #' @author Rafael Romero \email{rromero@@eio.upv.es}
-#' @references Pavia, JM, and Romero, R (2021). Improving estimates accuracy of voter transitions. Two new algorithms for ecological inference based on linear programming, mimeo.
+#' @references Pavia, JM, and Romero, R (2021). Improving estimates accuracy of voter transitions. Two new algorithms for ecological inference based on linear programming. \doi{10.31124/advance.14716638.v1}.
 #'
 #' @param votes_election1 data.frame (or matrix) of order IxJ (likely of final order IxJ-1
 #'                        in `regular` and `raw` scenarios) with the votes gained by the *J*
@@ -48,10 +48,18 @@
 #'                       Default, `"abs"`.
 #'                       If `distance.local = "abs"` lphom_local selects in its the second step the matrix
 #'                       closer to the temporary global solution under L_1 norm, among the first step compatible matrices.
-#'                       If `distance.local = "abs"` lphom_local selects in its the second step the matrix
+#'                       If `distance.local = "max"` lphom_local selects in its the second step the matrix
 #'                       closer to the temporary global solution under L_Inf norm, among the first step compatible matrices.
 #'                       If `distance.local = "none"`, the second step of lphom_local is not performed.
 #'
+#' @param counts A TRUE/FALSE value that indicates whether the problem is solved in integer values (counts) in
+#'               each iteration: zero (lphom) and intermediate and final (including unit) solutions.
+#'               If TRUE, the initial LP matrices are approximated in each iteration to the closest integer solution
+#'               solving the corresponding Integer Linear Program. Default, FALSE.
+#'
+#' @param solver A character string indicating the linear programming solver to be used, only
+#'               `lp_solve` and `symphony` are allowed. By default, `lp_solve`.
+#'
 #' @param burnin Number of initial solutions to be discarded before determining the final solution. By default, 0.
 #'
 #' @param verbose A TRUE/FALSE value that indicates if the main outputs of the function should be
@@ -102,14 +110,16 @@
 #'
 #' @return
 #' A list with the following components
-#'  \item{VTM}{ A matrix of order JxK with the estimated percentages of vote transitions from election 1 to election 2.}
+#'  \item{VTM}{ A matrix of order JxK with the estimated percentages of row-standardized vote transitions from election 1 to election 2.}
+#'  \item{VTM.votes}{ A matrix of order JxK with the estimated vote transitions from election 1 to election 2.}
 #'  \item{OTM}{ A matrix of order KxJ with the estimated percentages of the origin of the votes obtained for the different options of election 2.}
 #'  \item{HETe}{ The estimated heterogeneity index as defined in equation (15) of Pavia and Romero (2021).}
-#'  \item{VTM.complete}{ A matrix of order J'xK' with the estimated proportions of vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
+#'  \item{VTM.complete}{ A matrix of order J'xK' with the estimated proportions of row-standardized vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
+#'  \item{VTM.complete.votes}{ A matrix of order J'xK' with the estimated vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
 #'  \item{VTM.sequence}{ Array of order J'xK'x(iter+1) (where `iter` is the efective number of iterations performed) of the estimated matrices corresponding to each iteration.}
 #'  \item{HETe.sequence}{ Numeric vector of length `iter+1` with the `HETe` coefficients corresponding to the matrices in `VTM.sequence`.}
-#'  \item{VTM_units}{ An array of order J'xK'xI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit in the selected iteration.}
-#'  \item{VTM_votes}{ An array of order J'xK'xI with the estimated matrix of vote transitions from election 1 to election 2 attained for for each unit in the selected iteration.}
+#'  \item{VTM.prop.units}{ An array of order J'xK'xI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit in the selected iteration.}
+#'  \item{VTM.votes.units}{ An array of order J'xK'xI with the estimated matrix of vote transitions from election 1 to election 2 attained for for each unit in the selected iteration.}
 #'  \item{zeros}{ A list of vectors of length two, indicating the election options for which no transfer of votes are allowed between election 1 and election 2.}
 #'  \item{iter}{ The real final number of iterations performed before ending the process.}
 #'  \item{iter.min}{ Number of the iteration associated to the selected `VTM` solution.}
@@ -130,125 +140,137 @@
 #'
 #'
 #' @family linear programing ecological inference functions
-#' @seealso \code{\link{lphom}} \code{\link{tslphom}}
+#' @seealso \code{\link{lphom}} \code{\link{tslphom}} \code{\link{lclphom}}
 #'
 #' @examples
-#' mt.ns <- nslphom(France2017P[, 1:8] , France2017P[, 9:12],
-#'                  new_and_exit_voters= "raw")
+#' mt.ns <- nslphom(France2017P[, 1:8] , France2017P[, 9:12], new_and_exit_voters= "raw")
 #' mt.ns$VTM
 #' mt.ns$HETe
 #' mt.ns$solution_init$HETe_init
-#
-nslphom <- function(votes_election1, votes_election2,
+#'
+#' @importFrom Rsymphony Rsymphony_solve_LP
+#' @importFrom lpSolve lp
+#'
+
+nslphom <- function(votes_election1,
+                    votes_election2,
                     new_and_exit_voters = c("regular", "raw", "simultaneous", "full", "gold"),
-                    structural_zeros = NULL, iter.max = 10, min.first = FALSE,
-                    uniform = TRUE, distance.local = c("abs", "max", "none"),
-                    burnin = 0, verbose = FALSE, tol = 10^-5 ){
-
-  # Calculo de la solucion inicial
-  lphom_inic <- lphom(votes_election1 = votes_election1, votes_election2 = votes_election2,
-                      new_and_exit_voters = new_and_exit_voters, structural_zeros, verbose = FALSE)
-  lphom0 <- lphom_inic
+                    structural_zeros = NULL,
+                    iter.max = 10,
+                    min.first = FALSE,
+                    uniform = TRUE,
+                    distance.local = c("abs", "max", "none"),
+                    counts = FALSE,
+                    solver = "lp_solve",
+                    burnin = 0,
+                    verbose = FALSE,
+                    tol = 10^-5 ){
 
+  if (iter.max < 0 | iter.max%%1 > 0)
+    stop('iter.max must be a positive integer')
   if (!(distance.local[1] %in% c("abs", "max", "none")))
     stop('Not allowed string for argument "distance.local".
          The only allowed strings for "distance.local" are "abs", "max" and "none".')
 
-  # Funcion local a aplicar
-  if (uniform) {
-    if (distance.local[1] == "abs") {
-      lphom_unit <- lphom_local_abs
-    } else if (distance.local[1] == "max") {
-      lphom_unit <- lphom_local_max
-    } else if (distance.local[1] == "none") {
-      lphom_unit <- lphom_local
-    }
+  if (min.first){
+    burnin <- 0L
+    iter.max <- max(iter.max, 100L)
   } else {
-    if (distance.local[1] == "abs") {
-      lphom_unit <- calculo_MT_unidad_abs
-    } else if (distance.local[1] == "max") {
-      lphom_unit <- calculo_MT_unidad_max
-    } else if (distance.local[1] == "none") {
-      lphom_unit <- calculo_MT_unidad
-    }
+    if(iter.max <= burnin)
+      stop('The number of iterations (iter.max) must be higher than burnin')
   }
 
-  if (min.first){
-    burnin <- 0
-    iter.max <- max(iter.max, 100)
-  }
+  # Funcion local a aplicar
+  lphom_unit <- lp_solver_local(uniform = uniform,
+                                distance.local = distance.local)
+
+  # Calculo de la solucion inicial
+  lphom_inic <- lphom(votes_election1 = votes_election1, votes_election2 = votes_election2,
+                      new_and_exit_voters = new_and_exit_voters, structural_zeros,
+                      counts = counts, verbose = FALSE, solver = solver)
+  lphom0 <- lphom_inic
+
   zeros <- determinar_zeros_estructurales(lphom_inic)
 
   # Inicio proceso iterativo
-  VTM.sequence <- array(NA, c(dim(lphom_inic$VTM.complete), iter.max + 1))
-  VTM_votos.sequence <- array(NA, c(dim(lphom_inic$VTM.complete), iter.max + 1))
+  VTM.sequence <- array(NA, c(dim(lphom_inic$VTM.complete), iter.max + 1L))
+  VTM_votos.sequence <- array(NA, c(dim(lphom_inic$VTM.complete), iter.max + 1L))
   VTM_units.sequence <- array(NA, c(dim(lphom_inic$VTM.complete),
-                                    nrow(lphom_inic$origin), iter.max + 1))
+                                    nrow(lphom_inic$origin), iter.max + 1L))
   votos_units.sequence <- array(NA, c(dim(lphom_inic$VTM.complete),
-                                      nrow(lphom_inic$origin), iter.max + 1))
-  EHet.sequence <- array(NA, c(dim(lphom_inic$EHet), iter.max + 1))
-  iter <- 0
+                                      nrow(lphom_inic$origin), iter.max + 1L))
+  EHet.sequence <- array(NA, c(dim(lphom_inic$EHet), iter.max + 1L))
+  iter <- 0L
   dif.max <- Inf
   VTM.iter <- lphom0$VTM.complete <- lphom_inic$VTM.complete
-  VTM.sequence[, , iter+1] <- VTM.iter
+  VTM.sequence[, , iter + 1L] <- VTM.iter
   HETe.sequence <- lphom_inic$HETe
-  EHet.sequence[, , iter+1] <- lphom_inic$HETe
+  EHet.sequence[, , iter + 1L] <- lphom_inic$HETe
 
   while (iter < iter.max & dif.max > tol){
     # Calculo de las soluciones locales
     VTM_units <- votos_units <- array(NA, c(dim(lphom_inic$VTM.complete), nrow(lphom_inic$origin)))
-    for (i in 1:nrow(lphom_inic$origin)){
-      VTM_units[, , i] <- lphom_unit(lphom.object = lphom0, iii = i)
-      votos_units[, , i] <- VTM_units[, , i]/rowSums(VTM_units[, , i])*lphom_inic$origin[i, ]
-      VTM_units[lphom_inic$origin[i, ] == 0, , i] <- 0
+    for (i in 1L:nrow(lphom_inic$origin)){
+      VTM_units[, , i] <- lphom_unit(lphom.object = lphom0, iii = i, solver = solver)
+      if (counts){
+        votos_units[, , i] <-dec2counts(VTM_units[, , i]*lphom_inic$origin[i, ],
+                                        lphom_inic$origin[i,], lphom_inic$destination[i,])
+        VTM_units[, , i] <- votos_units[, , i]/rowSums(votos_units[, , i])
+      } else {
+        votos_units[, , i] <- VTM_units[, , i]/rowSums(VTM_units[, , i])*lphom_inic$origin[i, ]
+      }
+      VTM_units[lphom_inic$origin[i, ] == 0L, , i] <- 0L
     }
+    votos_units[is.na(votos_units)] <- 0L
     VTM_votos_homogeneos <- HET_MT.votos_MT.prop_Y(votos_units)
     VTM_votos <- VTM_votos_homogeneos$MT.votos
     VTM.complete <- VTM_votos_homogeneos$MT.pro
-    iter <- iter + 1
+    iter <- iter + 1L
     dif.max <- max(abs(VTM.complete - VTM.iter))
     #      dif0 <- max(VTM.complete - abs(lphom_inic$VTM.complete))
     #      print(iter)
     #      print(dif0)
     #      print(dif.max)
     VTM.iter <- lphom0$VTM.complete <- VTM.complete
-    VTM.sequence[, , iter+1] <- VTM.iter
+    VTM.sequence[, , iter + 1L] <- VTM.iter
     HETe.sequence <- c(HETe.sequence, VTM_votos_homogeneos$HET)
-    VTM_votos.sequence[, , iter+1] <- VTM_votos
-    VTM_units.sequence[, , , iter+1] <- VTM_units
-    votos_units.sequence[, , , iter+1] <- votos_units
-    EHet.sequence[, , iter+1] <- VTM_votos_homogeneos$EHet
-    if (min.first & (HETe.sequence[iter+1] > HETe.sequence[iter])) dif.max <- -Inf
+    VTM_votos.sequence[, , iter + 1L] <- VTM_votos
+    VTM_units.sequence[, , , iter + 1L] <- VTM_units
+    votos_units.sequence[, , , iter + 1L] <- votos_units
+    EHet.sequence[, , iter + 1L] <- VTM_votos_homogeneos$EHet
+    if (min.first & (HETe.sequence[iter + 1L] > HETe.sequence[iter])) dif.max <- -Inf
   } # End while
   dimnames(VTM.sequence) <- c(dimnames(lphom_inic$VTM.complete),
-                              list(paste0("iter = ", 0:iter.max)))
+                              list(paste0("iter = ", 0L:iter.max)))
   #  names(HETe.sequence) <- paste0("iter = ", 0:iter.max)
 
   # En caso de que converga antes de alcanzar el máximo de iteraciones
-  VTM.sequence <- VTM.sequence[, , 1:(iter+1)]
-  VTM_votos.sequence <- VTM_votos.sequence[, , 1:(iter+1)]
-  VTM_units.sequence <- VTM_units.sequence[, , , 1:(iter+1)]
-  votos_units.sequence <- votos_units.sequence[, , , 1:(iter+1)]
+  VTM.sequence <- VTM.sequence[, , 1L:(iter+1L)]
+  VTM_votos.sequence <- VTM_votos.sequence[, , 1L:(iter+1L)]
+  VTM_units.sequence <- VTM_units.sequence[, , , 1L:(iter+1L)]
+  votos_units.sequence <- votos_units.sequence[, , , 1L:(iter+1L)]
 
   # Solution
-  if (iter < burnin) burnin <- iter - 1
-  iter.select <- which.min(HETe.sequence[(burnin+2):(iter+1)])
-  VTM.complete <- VTM.sequence[, , burnin+1+iter.select]
-  VTM_votos <- VTM_votos.sequence[, , burnin+1+iter.select]
-  VTM_units <- VTM_units.sequence[, , , burnin+1+iter.select]
-  votos_units <- votos_units.sequence[, , , burnin+1+iter.select]
+  if (iter < burnin) burnin <- iter - 1L
+  iter.select <- which.min(HETe.sequence[(burnin+2L):(iter+1L)])
+  VTM.complete <- VTM.sequence[, , burnin+1L+iter.select]
+  VTM_votos <- VTM_votos.sequence[, , burnin+1L+iter.select]
+  VTM_units <- VTM_units.sequence[, , , burnin+1L+iter.select]
+  votos_units <- votos_units.sequence[, , , burnin+1L+iter.select]
   OTM <- round(t(VTM_votos)/colSums(VTM_votos)*100, 2)
-  OTM <- OTM[c(1:nrow(lphom_inic$OTM)), c(1:ncol(lphom_inic$OTM))]
-  EHet <- EHet.sequence[, , burnin+1+iter.select]
+  OTM <- OTM[c(1L:nrow(lphom_inic$OTM)), c(1L:ncol(lphom_inic$OTM))]
+  EHet <- EHet.sequence[, , burnin+1L+iter.select]
 
   # Improving the solution
   dimnames(OTM) <- dimnames(lphom_inic$OTM)
-  HETe <- HETe.sequence[burnin+1+iter.select]
+  HETe <- HETe.sequence[burnin+1L+iter.select]
   dimnames(VTM_units) <- dimnames(votos_units) <- c(dimnames(VTM.complete),
                                                     list(rownames(lphom_inic$origin)))
   dimnames(EHet) <- dimnames(lphom_inic$EHet)
+  dimnames(VTM_votos) <- dimnames(lphom_inic$VTM.complete)
 
-  VTM <- round(VTM.complete[c(1:nrow(lphom_inic$VTM)), c(1:ncol(lphom_inic$VTM))]*100, 2)
+  VTM <- round(VTM.complete[c(1L:nrow(lphom_inic$VTM)), c(1L:ncol(lphom_inic$VTM))]*100, 2)
+  VTM.votes <- VTM_votos[c(1L:nrow(lphom_inic$VTM)), c(1L:ncol(lphom_inic$VTM))]
 
   if (verbose){
     cat("\n\nEstimated Heterogeneity Index HETe:",round(HETe, 2),"%\n")
@@ -257,19 +279,19 @@ nslphom <- function(votes_election1, votes_election2,
     cat("\n\n Origin (%) of the votes obtained in Election 2\n\n")
     print(OTM)
   }
-  inputs <- lphom_inic$inputs
-  inputs$iter.max <- iter.max
-  inputs$min.first <- min.first
-  inputs$uniform <- uniform
-  inputs$distance.local <- distance.local
-  inputs$burnin <- burnin
-  inputs$tol <- tol
-  inic <- lphom_inic[c(1:4,8)]
+  lphom_inic$inputs$verbose <- verbose
+  inputs <- c(lphom_inic$inputs, "iter.max" = iter.max, "min.first" = min.first,
+              "uniform" <- uniform, "distance.local" = distance.local,
+              "burnin" = burnin, "tol" = tol)
+  inic <- lphom_inic[c(1L:6L, 10L)]
   names(inic) <- paste0(names(inic), "_init")
-  return(list("VTM" = VTM, "OTM" = OTM, "HETe" = HETe, "VTM.complete" = VTM.complete,
+  output <- list("VTM" = VTM, "VTM.votes" = VTM.votes, "OTM" = OTM, "HETe" = HETe,
+              "VTM.complete" = VTM.complete, "VTM.complete.votes" = VTM_votos,
               "VTM.sequence" = VTM.sequence, "HETe.sequence" = HETe.sequence,
-              "VTM_units" = VTM_units, "VTM_votes" = votos_units, "zeros" = zeros,
+              "VTM.prop.units" = VTM_units, "VTM.votes.units" = votos_units, "zeros" = zeros,
               "iter" = iter, "iter.min" = burnin + iter.select, "EHet" = EHet,
               "inputs" = inputs, "origin" = lphom_inic$origin,
-              "destination" = lphom_inic$destination, "solution_init" = inic))
+              "destination" = lphom_inic$destination, "solution_init" = inic)
+  class(output) <- "nslphom"
+  return(output)
 }
