@@ -4,7 +4,7 @@
 #'
 #' @author Jose M. Pavia, \email{pavia@@uv.es}
 #' @author Rafael Romero \email{rromero@@eio.upv.es}
-#' @references Pavia, JM, and Romero, R (2021). Improving estimates accuracy of voter transitions. Two new algorithms for ecological inference based on linear programming, mimeo.
+#' @references Pavia, JM, and Romero, R (2021). Improving estimates accuracy of voter transitions. Two new algorithms for ecological inference based on linear programming. \doi{10.31124/advance.14716638.v1}.
 #'
 #' @param votes_election1 data.frame (or matrix) of order IxJ (likely of final order IxJ-1
 #'                        in `regular` and `raw` scenarios) with the votes gained by the *J*
@@ -38,13 +38,21 @@
 #'                       Default, `"abs"`.
 #'                       If `distance.local = "abs"` lphom_local selects in its the second step the matrix
 #'                       closer to the **lphom()** solution under L_1 norm, among the first step compatible matrices.
-#'                       If `distance.local = "abs"` lphom_local selects in its the second step the matrix
+#'                       If `distance.local = "max"` lphom_local selects in its the second step the matrix
 #'                       closer to the **lphom()** solution under L_Inf norm, among the first step compatible matrices.
 #'                       If `distance.local = "none"`, the second step of lphom_local is not performed.
 #'
+#' @param counts A TRUE/FALSE value that indicates whether the problem is solved in integer values (counts),
+#'               in both iterations: zero (lphom) and final (including unit) solutions. If TRUE, the LP matrices
+#'               are approximated to the closest integer solution solving the corresponding Integer Linear Program.
+#'               Default, FALSE.
+#'
 #' @param verbose A TRUE/FALSE value that indicates if the main outputs of the function should be
 #'                printed on the screen. Default, FALSE.
 #'
+#' @param solver A character string indicating the linear programming solver to be used, only
+#'               `lp_solve` and `symphony` are allowed. By default, `lp_solve`.
+
 #'
 #' @details Description of the `new_and_exit_voters` argument in more detail.
 #' \itemize{
@@ -87,12 +95,14 @@
 #'
 #' @return
 #' A list with the following components
-#'  \item{VTM}{ A matrix of order JxK with the estimated percentages of vote transitions from election 1 to election 2.}
+#'  \item{VTM}{ A matrix of order JxK with the estimated percentages of row-standardized vote transitions from election 1 to election 2.}
+#'  \item{VTM.votes}{ A matrix of order JxK with the estimated vote transitions from election 1 to election 2.}
 #'  \item{OTM}{ A matrix of order KxJ with the estimated percentages of the origin of the votes obtained for the different options of election 2.}
 #'  \item{HETe}{ The estimated heterogeneity index as defined in equation (15) of Pavia and Romero (2021).}
-#'  \item{VTM.complete}{ A matrix of order J'xK' with the estimated proportions of vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
-#'  \item{VTM_units}{ An array of order J'xK'xI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
-#'  \item{VTM_votes}{ An array of order J'xK'xI with the estimated matrix of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
+#'  \item{VTM.complete}{ A matrix of order J'xK' with the estimated proportions of row-standardized vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
+#'  \item{VTM.complete.votes}{ A matrix of order J'xK' with the estimated vote transitions from election 1 to election 2, including in `regular` and `raw` scenarios the row and the column corresponding to net_entries and net_exits even when they are really small, less than 1% in all units.}
+#'  \item{VTM.prop.units}{ An array of order J'xK'xI with the estimated proportions of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
+#'  \item{VTM.votes.units}{ An array of order J'xK'xI with the estimated matrix of vote transitions from election 1 to election 2 attained for each unit after adjusting the **lphom()** initial estimate.}
 #'  \item{zeros}{ A list of vectors of length two, indicating the election options for which no transfer of votes are allowed between election 1 and election 2.}
 #'  \item{inputs}{ A list containing all the objects with the values used as arguments by the function.}
 #'  \item{origin}{ A matrix with the final data used as votes of the origin election after taking into account the level of information available regarding to new entries and exits of the election censuses between the two elections.}
@@ -112,71 +122,73 @@
 #'
 #'
 #' @family linear programing ecological inference functions
-#' @seealso \code{\link{lphom}} \code{\link{nslphom}}
+#' @seealso \code{\link{lphom}} \code{\link{nslphom}} \code{\link{lclphom}}
 #'
 #' @examples
-#' mt.ts <- tslphom(France2017P[, 1:8] , France2017P[, 9:12],
-#'                  new_and_exit_voters= "raw")
+#' mt.ts <- tslphom(France2017P[, 1:8] , France2017P[, 9:12], new_and_exit_voters= "raw")
 #' mt.ts$VTM
 #' mt.ts$HETe
 #' mt.ts$solution_init$HETe_init
+#'
+#' @importFrom Rsymphony Rsymphony_solve_LP
+#' @importFrom lpSolve lp
+#'
+
 #
-tslphom <- function(votes_election1, votes_election2,
+tslphom <- function(votes_election1,
+                    votes_election2,
                     new_and_exit_voters = c("regular", "raw", "simultaneous", "full", "gold"),
-                    structural_zeros = NULL, uniform = TRUE,
+                    structural_zeros = NULL,
+                    uniform = TRUE,
                     distance.local = c("abs", "max", "none"),
-                    verbose = FALSE){
+                    counts = FALSE,
+                    verbose = FALSE,
+                    solver = "lp_solve"){
 
   # Calculo de la solucion inicial
   lphom_inic <- lphom(votes_election1 = votes_election1, votes_election2 = votes_election2,
-                      new_and_exit_voters = new_and_exit_voters, structural_zeros, verbose = FALSE)
+                      new_and_exit_voters = new_and_exit_voters, structural_zeros,
+                      counts = counts, verbose = FALSE, solver = solver)
 
-  if (!(distance.local[1] %in% c("abs", "max", "none")))
+  if (!(distance.local[1L] %in% c("abs", "max", "none")))
     stop('Not allowed string for argument "distance.local".
          The only allowed strings for "distance.local" are "abs", "max" and "none".')
 
   # Funcion local a aplicar
-  if (uniform) {
-     if (distance.local[1] == "abs") {
-        lphom_unit <- lphom_local_abs
-     } else if (distance.local[1] == "max") {
-        lphom_unit <- lphom_local_max
-     } else if (distance.local[1] == "none") {
-        lphom_unit <- lphom_local
-     }
-  } else {
-     if (distance.local[1] == "abs") {
-        lphom_unit <- calculo_MT_unidad_abs
-     } else if (distance.local[1] == "max") {
-       lphom_unit <- calculo_MT_unidad_max
-     } else if (distance.local[1] == "none") {
-       lphom_unit <- calculo_MT_unidad
-     }
-  }
+  lphom_unit <- lp_solver_local(uniform = uniform,
+                                distance.local = distance.local)
 
   # Calculo de las soluciones locales
   zeros <- determinar_zeros_estructurales(lphom_inic)
   VTM_units <- votos_units <- array(NA, c(dim(lphom_inic$VTM.complete), nrow(lphom_inic$origin)))
 
-  for (i in 1:nrow(lphom_inic$origin)){
-      VTM_units[, , i] <- lphom_unit(lphom.object = lphom_inic, iii = i)
-      votos_units[, , i] <- VTM_units[, , i]/rowSums(VTM_units[, , i])*lphom_inic$origin[i, ]
-      VTM_units[lphom_inic$origin[i, ] == 0, , i] <- 0
+  for (i in 1L:nrow(lphom_inic$origin)){
+      VTM_units[, , i] <- lphom_unit(lphom.object = lphom_inic, iii = i, solver = solver)
+      if (counts){
+        votos_units[, , i] <-dec2counts(VTM_units[, , i]*lphom_inic$origin[i, ],
+                                        lphom_inic$origin[i,], lphom_inic$destination[i,])
+        VTM_units[, , i] <- votos_units[, , i]/rowSums(votos_units[, , i])
+      } else {
+        votos_units[, , i] <- VTM_units[, , i]/rowSums(VTM_units[, , i])*lphom_inic$origin[i, ]
+      }
+      VTM_units[lphom_inic$origin[i, ] == 0L, , i] <- 0L
   }
-
+  votos_units[is.na(votos_units)] <- 0L
   VTM_votos_homogeneos <- HET_MT.votos_MT.prop_Y(votos_units)
   VTM_votos <- VTM_votos_homogeneos$MT.votos
   VTM.complete <- VTM_votos_homogeneos$MT.pro
   EHet <- VTM_votos_homogeneos$EHet
   dimnames(VTM.complete) <- dimnames(lphom_inic$VTM.complete)
+  dimnames(VTM_votos) <- dimnames(lphom_inic$VTM.complete)
   OTM <- round(t(VTM_votos)/colSums(VTM_votos)*100, 2)
-  OTM <- OTM[c(1:nrow(lphom_inic$OTM)), c(1:ncol(lphom_inic$OTM))]
+  OTM <- OTM[c(1L:nrow(lphom_inic$OTM)), c(1L:ncol(lphom_inic$OTM))]
   dimnames(OTM) <- dimnames(lphom_inic$OTM)
   HETe <- VTM_votos_homogeneos$HET
   dimnames(VTM_units) <- dimnames(votos_units) <- c(dimnames(VTM.complete),
                                                     list(rownames(lphom_inic$origin)))
   dimnames(EHet) <- dimnames(lphom_inic$EHet)
-  VTM <- round(VTM.complete[c(1:nrow(lphom_inic$VTM)), c(1:ncol(lphom_inic$VTM))]*100, 2)
+  VTM <- round(VTM.complete[c(1L:nrow(lphom_inic$VTM)), c(1L:ncol(lphom_inic$VTM))]*100, 2)
+  VTM.votes <- VTM_votos[c(1L:nrow(lphom_inic$VTM)), c(1L:ncol(lphom_inic$VTM))]
 
   if (verbose){
     cat("\n\nEstimated Heterogeneity Index HETe:",round(HETe, 2),"%\n")
@@ -185,13 +197,15 @@ tslphom <- function(votes_election1, votes_election2,
     cat("\n\n Origin (%) of the votes obtained in Election 2\n\n")
     print(OTM)
   }
-  inputs <- lphom_inic$inputs
-  inputs$uniform <- uniform
-  inputs$distance.local <- distance.local
-  inic <- lphom_inic[c(1:4,8)]
+  lphom_inic$inputs$verbose <- verbose
+  inputs <- c(lphom_inic$inputs, "uniform" = uniform, "distance.local" = distance.local)
+  inic <- lphom_inic[c(1L:6L, 10L)]
   names(inic) <- paste0(names(inic), "_init")
-  return(list("VTM" = VTM, "OTM" = OTM, "HETe" = HETe, "VTM.complete" = VTM.complete,
-              "VTM_units" = VTM_units, "VTM_votes" = votos_units, "zeros" = zeros,
-              "inputs" = inputs, "origin" = lphom_inic$origin,
-              "destination" = lphom_inic$destination, "EHet" = EHet, "solution_init" = inic))
+  output <- list("VTM" = VTM, "VTM.votes" = VTM.votes, "OTM" = OTM, "HETe" = HETe,
+                 "VTM.complete" = VTM.complete, "VTM.complete.votes" = VTM_votos,
+                 "VTM.prop.units" = VTM_units, "VTM.votes.units" = votos_units, "zeros" = zeros,
+                 "inputs" = inputs, "origin" = lphom_inic$origin,
+                 "destination" = lphom_inic$destination, "EHet" = EHet, "solution_init" = inic)
+  class(output) <- "tslphom"
+  return(output)
 }
